/*
 * Copyright (C) 2009 Apple Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer. 
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution. 
 * 3.  Neither the name of Apple Computer, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

.webkit-css-comment { 
    color: rgb(0, 116, 0);
}

.webkit-css-url, .webkit-css-color, .webkit-css-string, .webkit-css-keyword {
    color: rgb(7, 144, 154);
 }

.webkit-css-number {
    color: rgb(50, 0, 255);
}

.webkit-css-property, .webkit-css-at-rule {
    color: rgb(200, 0, 0);
}

.webkit-css-selector {
    color: black;
}

.webkit-css-important {
    color: rgb(200, 0, 180);
}

.webkit-javascript-comment {
    color: rgb(0, 116, 0);
}

.webkit-javascript-keyword {
    color: rgb(170, 13, 145);
}

.webkit-javascript-number {
    color: rgb(28, 0, 207);
}

.webkit-javascript-string, .webkit-javascript-regexp {
    color: rgb(196, 26, 22);
}

.webkit-javascript-ident {
    color: black;
}

.webkit-html-comment {
    /* Keep this in sync with view-source.css (.webkit-html-comment) */
    color: rgb(35, 110, 37);
}

.webkit-html-tag {
    /* Keep this in sync with view-source.css (.webkit-html-tag) */
    color: rgb(136, 18, 128);
}

.webkit-html-doctype {
    /* Keep this in sync with view-source.css (.webkit-html-doctype) */
    color: rgb(192, 192, 192);
}

.webkit-html-attribute-name {
    /* Keep this in sync with view-source.css (.webkit-html-attribute-name) */
    color: rgb(153, 69, 0);
}

.webkit-html-attribute-value {
    /* Keep this in sync with view-source.css (.webkit-html-attribute-value) */
    color: rgb(26, 26, 166);
}

.webkit-html-external-link, .webkit-html-resource-link {
    /* Keep this in sync with view-source.css (.webkit-html-external-link, .webkit-html-resource-link) */
    color: #00e;
}

.webkit-html-external-link {
    /* Keep this in sync with view-source.css (.webkit-html-external-link) */
    text-decoration: none;
}

.webkit-html-external-link:hover {
    /* Keep this in sync with view-source.css (.webkit-html-external-link:hover) */
    text-decoration: underline;
}
