var localizedStrings = new Object;

localizedStrings[" (%d)"] = " (%d)";
localizedStrings[" (repeated %d times)"] = " (repeated %d times)";
localizedStrings["1 - %d"] = "1 - %d";
localizedStrings["%.0fB"] = "%.0fB";
localizedStrings["%.0fKB"] = "%.0fKB";
localizedStrings["%.0fMB"] = "%.0fMB";
localizedStrings["%.0fms"] = "%.0fms";
localizedStrings["%.1f days"] = "%.1f days";
localizedStrings["%.1fhrs"] = "%.1fhrs";
localizedStrings["%.1fmin"] = "%.1fmin";
localizedStrings["%.2f%%"] = "%.2f%%";
localizedStrings["%.2fKB"] = "%.2fKB";
localizedStrings["%.2fMB"] = "%.2fMB";
localizedStrings["%.2fs"] = "%.2fs";
localizedStrings["%.3fms"] = "%.3fms";
localizedStrings["%d console messages are not shown."] = "%d console messages are not shown.";
localizedStrings["%d cookies (%s)"] = "%d cookies (%s)";
localizedStrings["%d error"] = "%d error";
localizedStrings["%d error, %d warning"] = "%d error, %d warning";
localizedStrings["%d error, %d warnings"] = "%d error, %d warnings";
localizedStrings["%d errors"] = "%d errors";
localizedStrings["%d errors, %d warning"] = "%d errors, %d warning";
localizedStrings["%d errors, %d warnings"] = "%d errors, %d warnings";
localizedStrings["%d matches"] = "%d matches";
localizedStrings["%d of %d captured records are visible"] = "%d of %d captured records are visible";
localizedStrings["%d warning"] = "%d warning";
localizedStrings["%d warnings"] = "%d warnings";
localizedStrings["%d × %d pixels (Natural: %d × %d pixels)"] = "%d × %d pixels (Natural: %d × %d pixels)";
localizedStrings["%d × %d pixels"] = "%d × %d pixels";
localizedStrings["%d × %d"] = "%d × %d";
localizedStrings["%s (%s)"] = "%s (%s)";
localizedStrings["%s (at %s)"] = "%s (at %s)";
localizedStrings["%s (from cache)"] = "%s (from cache)";
localizedStrings["%s - Details"] = "%s - Details";
localizedStrings["%s >1000%%"] = "%s >1000%%";
localizedStrings["%s collected"] = "%s collected";
localizedStrings["%s download"] = "%s download";
localizedStrings["%s latency"] = "%s latency";
localizedStrings["%s latency, %s download (%s total)"] = "%s latency, %s download (%s total)";
localizedStrings["%s of %s"] = "%s of %s";
localizedStrings["%s%.2f%%"] = "%s%.2f%%";
localizedStrings["%s%d"] = "%s%d";
localizedStrings["%s%s"] = "%s%s";
localizedStrings["(anonymous function)"] = "(anonymous function)";
localizedStrings["(internal script)"] = "(internal script)";
localizedStrings["(from cache)"] = "(from cache)";
localizedStrings["(program)"] = "(program)";
localizedStrings["(text)"] = "(text)";
localizedStrings["(unable to decode value)"] = "(unable to decode value)";
localizedStrings["(whitespace)"] = "(whitespace)";
localizedStrings["1 match"] = "1 match";
localizedStrings["Application Cache"] = "Application Cache";
localizedStrings["Accept suggestion"] = "Accept suggestion";
localizedStrings["Activate all breakpoints."] = "Activate all breakpoints.";
localizedStrings["Add Attribute"] = "Add Attribute";
localizedStrings["Add Breakpoint"] = "Add Breakpoint";
localizedStrings["Add Conditional Breakpoint…"] = "Add Conditional Breakpoint…";
localizedStrings["Add New"] = "Add New";
localizedStrings["Add"] = "Add";
localizedStrings["Aggregated Time"] = "Aggregated Time";
localizedStrings["All Nodes"] = "All Nodes";
localizedStrings["All Panels"] = "All Panels";
localizedStrings["All"] = "All";
localizedStrings["Allow debugging workers. Enabling this option will replace native workers with the iframe-based JavaScript implementation"] = "Allow debugging workers. Enabling this option will replace native workers with the iframe-based JavaScript implementation";
localizedStrings["Always enable"] = "Always enable";
localizedStrings["An error occurred trying to\nread the “%s” table."] = "An error occurred trying to\nread the “%s” table.";
localizedStrings["An unexpected error %s occurred."] = "An unexpected error %s occurred.";
localizedStrings["Any XHR"] = "Any XHR";
localizedStrings["As Authored"] = "As Authored";
localizedStrings["Assertion failed:"] = "Assertion failed:";
localizedStrings["Attribute Modified"] = "Attribute Modified";
localizedStrings["Audit Present State"] = "Audit Present State";
localizedStrings["Audits"] = "Audits";
localizedStrings["Average"] = "Average";
localizedStrings["Blocking"] = "Blocking";
localizedStrings["Breakpoints"] = "Breakpoints";
localizedStrings["Cookies"] = "Cookies";
localizedStrings["CPU PROFILES"] = "CPU PROFILES";
localizedStrings["Call Site stack"] = "Call Site stack";
localizedStrings["Call Stack"] = "Call Stack";
localizedStrings["Calls"] = "Calls";
localizedStrings["Clear all profiles."] = "Clear all profiles.";
localizedStrings["Clear Console"] = "Clear Console";
localizedStrings["Clear audit results."] = "Clear audit results.";
localizedStrings["Clear console log."] = "Clear console log.";
localizedStrings["Clear"] = "Clear";
localizedStrings["Click to change color format"] = "Click to change color format";
localizedStrings["Closure"] = "Closure";
localizedStrings["Code"] = "Code";
localizedStrings["Collected"] = "Collected";
localizedStrings["Compared to %s"] = "Compared to %s";
localizedStrings["Computed Style"] = "Computed Style";
localizedStrings["Connecting"] = "Connecting";
localizedStrings["Console"] = "Console";
localizedStrings["Constructor"] = "Constructor";
localizedStrings["Content"] = "Content";
localizedStrings["Continue to Here"] = "Continue to Here";
localizedStrings["Continue"] = "Continue";
localizedStrings["Control CPU profiling by pressing the %s button on the status bar."] = "Control CPU profiling by pressing the %s button on the status bar.";
localizedStrings["Copy as HTML"] = "Copy as HTML";
localizedStrings["Copy entry as HAR"] = "Copy entry as HAR";
localizedStrings["Copy network log as HAR"] = "Copy network log as HAR";
localizedStrings["Count"] = "Count";
localizedStrings["Databases"] = "Databases";
localizedStrings["DNS Lookup"] = "DNS Lookup";
localizedStrings["DOMContent event fired"] = "DOMContent event fired";
localizedStrings["DOMContent event"] = "DOMContent event";
localizedStrings["Database no longer has expected version."] = "Database no longer has expected version.";
localizedStrings["Database not found."] = "Database not found.";
localizedStrings["Deactivate all breakpoints."] = "Deactivate all breakpoints.";
localizedStrings["Debug"] = "Debug";
localizedStrings["Debugging disabled. Click to enable."] = "Debugging disabled. Click to enable.";
localizedStrings["Debugging enabled. Click to disable."] = "Debugging enabled. Click to disable.";
localizedStrings["Delete Node"] = "Delete Node";
localizedStrings["Delete watch expression."] = "Delete watch expression.";
localizedStrings["Delete"] = "Delete";
localizedStrings["Details"] = "Details";
localizedStrings["Dimensions"] = "Dimensions";
localizedStrings["Disable Breakpoint"] = "Disable Breakpoint";
localizedStrings["Dock to main window."] = "Dock to main window.";
localizedStrings["Document"] = "Document";
localizedStrings["Documents"] = "Documents";
localizedStrings["Domain"] = "Domain";
localizedStrings["Don't pause on exceptions.\nClick to Pause on all exceptions."] = "Don't pause on exceptions.\nClick to Pause on all exceptions.";
localizedStrings["Double-Click to toggle between URL encoded and decoded formats"] = "Double-Click to toggle between URL encoded and decoded formats";
localizedStrings["Duration"] = "Duration";
localizedStrings["Edit Attribute"] = "Edit Attribute";
localizedStrings["Edit Breakpoint…"] = "Edit Breakpoint…";
localizedStrings["Edit Text"] = "Edit Text";
localizedStrings["Edit as HTML"] = "Edit as HTML";
localizedStrings["Edit attribute"] = "Edit attribute";
localizedStrings["Edit"] = "Edit";
localizedStrings["Elements Panel"] = "Elements Panel";
localizedStrings["Elements"] = "Elements";
localizedStrings["Enable Breakpoint"] = "Enable Breakpoint";
localizedStrings["Enable Debugging"] = "Enable Debugging";
localizedStrings["Enable Profiling"] = "Enable Profiling";
localizedStrings["Enabling debugging will make scripts run slower."] = "Enabling debugging will make scripts run slower.";
localizedStrings["Enabling profiling will make scripts run slower."] = "Enabling profiling will make scripts run slower.";
localizedStrings["Errors"] = "Errors";
localizedStrings["Evaluate Script"] = "Evaluate Script";
localizedStrings["Event Document"] = "Event Document";
localizedStrings["Event Listeners"] = "Event Listeners";
localizedStrings["Event Target"] = "Event Target";
localizedStrings["Event"] = "Event";
localizedStrings["Exclude selected function."] = "Exclude selected function.";
localizedStrings["Execute command"] = "Execute command";
localizedStrings["Expand/collapse"] = "Expand/collapse";
localizedStrings["Expected Content Length"] = "Expected Content Length";
localizedStrings["Expires"] = "Expires";
localizedStrings["File size"] = "File size";
localizedStrings["Find next/previous"] = "Find next/previous";
localizedStrings["Finish Loading"] = "Finish Loading";
localizedStrings["Focus selected function."] = "Focus selected function.";
localizedStrings["Font"] = "Font";
localizedStrings["Fonts"] = "Fonts";
localizedStrings["Form Data"] = "Form Data";
localizedStrings["Function Call"] = "Function Call";
localizedStrings["Function"] = "Function";
localizedStrings["GC Event"] = "GC Event";
localizedStrings["Get a heap snapshot by pressing the %s button on the status bar."] = "Get a heap snapshot by pressing the %s button on the status bar.";
localizedStrings["Global"] = "Global";
localizedStrings["Go"] = "Go";
localizedStrings["Go to line: "] = "Go to line: ";
localizedStrings["HEAP SNAPSHOTS"] = "HEAP SNAPSHOTS";
localizedStrings["HSL Colors"] = "HSL Colors";
localizedStrings["HTTP"] = "HTTP";
localizedStrings["Headers"] = "Headers";
localizedStrings["Heavy (Bottom Up)"] = "Heavy (Bottom Up)";
localizedStrings["Hex Colors"] = "Hex Colors";
localizedStrings["Hide console."] = "Hide console.";
localizedStrings["Hide the records that are shorter than %s"] = "Hide the records that are shorter than %s";
localizedStrings["Image"] = "Image";
localizedStrings["Images"] = "Images";
localizedStrings["Increment/decrement by %f"] = "Increment/decrement by %f";
localizedStrings["Increment/decrement value"] = "Increment/decrement value";
localizedStrings["Inherited from"] = "Inherited from";
localizedStrings["Install Timer"] = "Install Timer";
localizedStrings["Key"] = "Key";
localizedStrings["Keyboard Shortcuts"] = "Keyboard Shortcuts";
localizedStrings["Local Storage"] = "Local Storage";
localizedStrings["Layout"] = "Layout";
localizedStrings["Load event fired"] = "Load event fired";
localizedStrings["Load event"] = "Load event";
localizedStrings["Loading (%d of %d)"] = "Loading (%d of %d)";
localizedStrings["Loading"] = "Loading";
localizedStrings["Local Files"] = "Local Files";
localizedStrings["Local"] = "Local";
localizedStrings["Location"] = "Location";
localizedStrings["Logs"] = "Logs";
localizedStrings["MIME Type"] = "MIME Type";
localizedStrings["MIME type"] = "MIME type";
localizedStrings["Mark"] = "Mark";
localizedStrings["Matched CSS Rules"] = "Matched CSS Rules";
localizedStrings["Memory"] = "Memory";
localizedStrings["Metrics"] = "Metrics";
localizedStrings["Name"] = "Name";
localizedStrings["Navigate elements"] = "Navigate elements";
localizedStrings["New Style Rule"] = "New Style Rule";
localizedStrings["Next/previous call frame"] = "Next/previous call frame";
localizedStrings["Next/previous command"] = "Next/previous command";
localizedStrings["Next/previous line"] = "Next/previous line";
localizedStrings["Next/previous panel"] = "Next/previous panel";
localizedStrings["Next/previous property"] = "Next/previous property";
localizedStrings["Next/previous suggestion"] = "Next/previous suggestion";
localizedStrings["No Application Cache information available."] = "No Application Cache information available.";
localizedStrings["No Breakpoints"] = "No Breakpoints";
localizedStrings["No Event Listeners"] = "No Event Listeners";
localizedStrings["No Properties"] = "No Properties";
localizedStrings["No Variables"] = "No Variables";
localizedStrings["No audits to run"] = "No audits to run";
localizedStrings["Node Removed"] = "Node Removed";
localizedStrings["Not Found"] = "Not Found";
localizedStrings["Not Paused"] = "Not Paused";
localizedStrings["Objects"] = "Objects";
localizedStrings["Offline"] = "Offline";
localizedStrings["Online"] = "Online";
localizedStrings["Only enable for this session"] = "Only enable for this session";
localizedStrings["Open Link in New Tab"] = "Open Link in New Tab";
localizedStrings["Open Link in New Window"] = "Open Link in New Window";
localizedStrings["Open Link in Resources Panel"] = "Open Link in Resources Panel";
localizedStrings["Other"] = "Other";
localizedStrings["Paint"] = "Paint";
localizedStrings["Parse"] = "Parse";
localizedStrings["Path"] = "Path";
localizedStrings["Pause on all exceptions.\nClick to Pause on uncaught exceptions."] = "Pause on all exceptions.\nClick to Pause on uncaught exceptions.";
localizedStrings["Pause on uncaught exceptions.\nClick to Not pause on exceptions."] = "Pause on uncaught exceptions.\nClick to Not pause on exceptions.";
localizedStrings["Pause script execution."] = "Pause script execution.";
localizedStrings["Paused"] = "Paused";
localizedStrings["Pausing"] = "Pausing";
localizedStrings["Profile %d"] = "Profile %d";
localizedStrings["Profiles"] = "Profiles";
localizedStrings["Profiling disabled. Click to enable."] = "Profiling disabled. Click to enable.";
localizedStrings["Profiling enabled. Click to disable."] = "Profiling enabled. Click to disable.";
localizedStrings["Properties"] = "Properties";
localizedStrings["Proxy"] = "Proxy";
localizedStrings["Pseudo ::%s element"] = "Pseudo ::%s element";
localizedStrings["Pseudo element"] = "Pseudo element";
localizedStrings["Query String Parameters"] = "Query String Parameters";
localizedStrings["RECORDS"] = "RECORDS";
localizedStrings["RESULTS"] = "RESULTS";
localizedStrings["RGB Colors"] = "RGB Colors";
localizedStrings["Recalculate Style"] = "Recalculate Style";
localizedStrings["Receive Data"] = "Receive Data";
localizedStrings["Receive Response"] = "Receive Response";
localizedStrings["Receiving"] = "Receiving";
localizedStrings["Record"] = "Record";
localizedStrings["Recording…"] = "Recording…";
localizedStrings["Refresh"] = "Refresh";
localizedStrings["Remove Breakpoint"] = "Remove Breakpoint";
localizedStrings["Remove Timer"] = "Remove Timer";
localizedStrings["Rendering"] = "Rendering";
localizedStrings["Repeats"] = "Repeats";
localizedStrings["Request Cookies"] = "Request Cookies";
localizedStrings["Request Headers"] = "Request Headers";
localizedStrings["Request Method"] = "Request Method";
localizedStrings["Request Payload"] = "Request Payload";
localizedStrings["Request URL"] = "Request URL";
localizedStrings["Resource interpreted as %s but transferred with MIME type %s."] = "Resource interpreted as %s but transferred with MIME type %s.";
localizedStrings["Resource"] = "Resource";
localizedStrings["Resources"] = "Resources";
localizedStrings["Response Cookies"] = "Response Cookies";
localizedStrings["Response Headers"] = "Response Headers";
localizedStrings["Restore all functions."] = "Restore all functions.";
localizedStrings["Reveal in Elements Panel"] = "Reveal in Elements Panel";
localizedStrings["Run %d"] = "Run %d";
localizedStrings["Run"] = "Run";
localizedStrings["Session Storage"] = "Session Storage";
localizedStrings["SSL"] = "SSL";
localizedStrings["Schedule Request"] = "Schedule Request";
localizedStrings["Scope Variables"] = "Scope Variables";
localizedStrings["Script"] = "Script";
localizedStrings["Scripting"] = "Scripting";
localizedStrings["Scripts Panel"] = "Scripts Panel";
localizedStrings["Scripts"] = "Scripts";
localizedStrings["Search %s"] = "Search %s";
localizedStrings["Search"] = "Search";
localizedStrings["Secure"] = "Secure";
localizedStrings["Select All"] = "Select All";
localizedStrings["Select an element in the page to inspect it."] = "Select an element in the page to inspect it.";
localizedStrings["Select audits to run"] = "Select audits to run";
localizedStrings["Selected Node Only"] = "Selected Node Only";
localizedStrings["Self Time"] = "Self Time";
localizedStrings["Self"] = "Self";
localizedStrings["Send Request"] = "Send Request";
localizedStrings["Sending"] = "Sending";
localizedStrings["Session"] = "Session";
localizedStrings["Show All Nodes (%d More)"] = "Show All Nodes (%d More)";
localizedStrings["Show absolute counts and sizes."] = "Show absolute counts and sizes.";
localizedStrings["Show absolute total and self times."] = "Show absolute total and self times.";
localizedStrings["Show console."] = "Show console.";
localizedStrings["Show counts and sizes as percentages."] = "Show counts and sizes as percentages.";
localizedStrings["Show inherited"] = "Show inherited";
localizedStrings["Show the next script resource."] = "Show the next script resource.";
localizedStrings["Show the previous script resource."] = "Show the previous script resource.";
localizedStrings["Show the records that are shorter than %s"] = "Show the records that are shorter than %s";
localizedStrings["Show total and self times as percentages."] = "Show total and self times as percentages.";
localizedStrings["Size"] = "Size";
localizedStrings["Snapshot %d"] = "Snapshot %d";
localizedStrings["End Time"] = "End Time";
localizedStrings["Latency"] = "Latency";
localizedStrings["Response Time"] = "Response Time";
localizedStrings["Start Time"] = "Start Time";
localizedStrings["Start profiling."] = "Start profiling.";
localizedStrings["Status Code"] = "Status Code";
localizedStrings["Step into next function call."] = "Step into next function call.";
localizedStrings["Step into"] = "Step into";
localizedStrings["Step out of current function."] = "Step out of current function.";
localizedStrings["Step out"] = "Step out";
localizedStrings["Step over next function call."] = "Step over next function call.";
localizedStrings["Step over"] = "Step over";
localizedStrings["Stepping"] = "Stepping";
localizedStrings["Stop profiling."] = "Stop profiling.";
localizedStrings["Style Attribute"] = "Style Attribute";
localizedStrings["Styles Pane"] = "Styles Pane";
localizedStrings["Styles"] = "Styles";
localizedStrings["Stylesheet"] = "Stylesheet";
localizedStrings["Stylesheets"] = "Stylesheets";
localizedStrings["Subtree Modified"] = "Subtree Modified";
localizedStrings["Take heap snapshot."] = "Take heap snapshot.";
localizedStrings["The breakpoint on line %d will stop only if this expression is true:"] = "The breakpoint on line %d will stop only if this expression is true:";
localizedStrings["The “%s”\ntable is empty."] = "The “%s”\ntable is empty.";
localizedStrings["This site has no cookies."] = "This site has no cookies.";
localizedStrings["This request has no detailed timing info."] = "This request has no detailed timing info.";
localizedStrings["Time"] = "Time";
localizedStrings["Timeline"] = "Timeline";
localizedStrings["Timelines"] = "Timelines";
localizedStrings["Timeout"] = "Timeout";
localizedStrings["Timer Fired"] = "Timer Fired";
localizedStrings["Timer ID"] = "Timer ID";
localizedStrings["Timing"] = "Timing";
localizedStrings["Toggle console"] = "Toggle console";
localizedStrings["Total"] = "Total";
localizedStrings["Tree (Top Down)"] = "Tree (Top Down)";
localizedStrings["Type"] = "Type";
localizedStrings["Undock into separate window."] = "Undock into separate window.";
localizedStrings["Use large resource rows."] = "Use large resource rows.";
localizedStrings["Use small resource rows."] = "Use small resource rows.";
localizedStrings["Used Heap Size"] = "Used Heap Size";
localizedStrings["Value"] = "Value";
localizedStrings["Waiting"] = "Waiting";
localizedStrings["Warnings"] = "Warnings";
localizedStrings["Watch Expressions"] = "Watch Expressions";
localizedStrings["Welcome to the Profiles panel"] = "Welcome to the Profiles panel";
localizedStrings["With Block"] = "With Block";
localizedStrings["Word Wrap"] = "Word Wrap";
localizedStrings["Workers"] = "Workers";
localizedStrings["XHR Load"] = "XHR Load";
localizedStrings["XHR Ready State Change"] = "XHR Ready State Change";
localizedStrings["XHR"] = "XHR";
localizedStrings["XMLHttpRequest logging"] = "XMLHttpRequest logging";
localizedStrings["You could save bandwidth by having your web server compress this transfer with gzip or zlib."] = "You could save bandwidth by having your web server compress this transfer with gzip or zlib.";
localizedStrings["You need to enable debugging before you can use the Scripts panel."] = "You need to enable debugging before you can use the Scripts panel.";
localizedStrings["You need to enable profiling before you can use the Profiles panel."] = "You need to enable profiling before you can use the Profiles panel.";
localizedStrings["[empty domain]"] = "[empty domain]";
localizedStrings["\xb1 Count"] = "\xb1 Count";
localizedStrings["\xb1 Size"] = "\xb1 Size";
localizedStrings["border"] = "border";
localizedStrings["content"] = "content";
localizedStrings["deleted"] = "deleted";
localizedStrings["line %d"] = "line %d";
localizedStrings["margin"] = "margin";
localizedStrings["new"] = "new";
localizedStrings["or"] = "or";
localizedStrings["padding"] = "padding";
localizedStrings["position"] = "position";
localizedStrings["user agent stylesheet"] = "user agent stylesheet";
localizedStrings["user stylesheet"] = "user stylesheet";
localizedStrings["via inspector"] = "via inspector";
localizedStrings["Break on Attributes Modifications"] = "Break on Attributes Modifications";
localizedStrings["Break on Node Removal"] = "Break on Node Removal";
localizedStrings["Break on Subtree Modifications"] = "Break on Subtree Modifications";
localizedStrings["<node>"] = "<node>";
localizedStrings["Paused on a \"%s\" breakpoint set on %s, because a new child was added to its descendant %s."] = "Paused on a \"%s\" breakpoint set on %s, because a new child was added to its descendant %s.";
localizedStrings["Paused on a \"%s\" breakpoint set on %s, because a new child was added to that node."] = "Paused on a \"%s\" breakpoint set on %s, because a new child was added to that node.";
localizedStrings["Paused on a \"%s\" breakpoint set on %s, because its descendant %s was removed."] = "Paused on a \"%s\" breakpoint set on %s, because its descendant %s was removed.";
localizedStrings["Paused on a \"%s\" breakpoint set on %s."] = "Paused on a \"%s\" breakpoint set on %s.";
localizedStrings["Paused on a XMLHttpRequest."] = "Paused on a XMLHttpRequest.";
localizedStrings["Paused on a \"%s\" Event Listener."] = "Paused on a \"%s\" Event Listener.";
localizedStrings["Paused on a JavaScript breakpoint."] = "Paused on a JavaScript breakpoint.";
localizedStrings["URL contains \"%s\""] = "URL contains \"%s\"";
localizedStrings["Network"] = "Network";
localizedStrings["Status"] = "Status";
localizedStrings["Method"] = "Method";
localizedStrings["Pending"] = "Pending";
localizedStrings["%d requests"] = "%d requests";
localizedStrings["%s transferred"] = "%s transferred";
localizedStrings["%s (onload: %s, DOMContentLoaded: %s)"] = "%s (onload: %s, DOMContentLoaded: %s)";
localizedStrings["DOM Breakpoints"] = "DOM Breakpoints";
localizedStrings["Event Listener Breakpoints"] = "Event Listener Breakpoints";
localizedStrings["XHR Breakpoints"] = "XHR Breakpoints";
localizedStrings["Mouse"] = "Mouse";
localizedStrings["Keyboard"] = "Keyboard";
localizedStrings["Clipboard"] = "Clipboard";
localizedStrings["DOM Mutation"] = "DOM Mutation";
localizedStrings["Device"] = "Device";
localizedStrings["Load"] = "Load";
localizedStrings["Control"] = "Control";
localizedStrings["Timer"] = "Timer";
localizedStrings["Set Timer"] = "Set Timer";
localizedStrings["Clear Timer"] = "Clear Timer";
localizedStrings["No requests captured. Reload the page to see detailed information on the network activity."] = "No requests captured. Reload the page to see detailed information on the network activity.";
localizedStrings["This request has no cookies."] = "This request has no cookies.";
localizedStrings["(original)"] = "(original)";
localizedStrings["(%d, %d)"] = "(%d, %d)";
localizedStrings["Revert to this revision"] = "Revert to this revision";
localizedStrings["WebSockets"] = "WebSockets";
localizedStrings["Frames"] = "Frames";
localizedStrings["Text"] = "Text";
localizedStrings["Transfer"] = "Transfer";
localizedStrings["Preserve Log upon Navigation"] = "Preserve Log upon Navigation";
localizedStrings["Go to Line"] = "Go to Line";
localizedStrings["URL"] = "URL";
localizedStrings["Format script."] = "Format script.";
localizedStrings["WebSocket"] = "WebSocket";
localizedStrings["(system)"] = "(system)";
localizedStrings["(compiled code)"] = "(compiled code)";
localizedStrings["Loading…"] = "Loading…";
localizedStrings["Parsing…"] = "Parsing…";
localizedStrings["Snapshotting…"] = "Snapshotting…";
localizedStrings["Please wait…"] = "Please wait…";
localizedStrings["Cancel"] = "Cancel";
localizedStrings["Show next %d"] = "Show next %d";
localizedStrings["Show all %d"] = "Show all %d";
localizedStrings["Comparison"] = "Comparison";
localizedStrings["#"] = "#";
localizedStrings["Retaining path"] = "Retaining path";
localizedStrings["Freed Size"] = "Freed Size";
localizedStrings["Object"] = "Object";
localizedStrings["Alloc. Size"] = "Alloc. Size";
localizedStrings["Summary"] = "Summary";
localizedStrings["Dominators"] = "Dominators";
localizedStrings["Length"] = "Length";
localizedStrings["# Deleted"] = "# Deleted";
localizedStrings["Containment"] = "Containment";
localizedStrings["Retained Size"] = "Retained Size";
localizedStrings["Shallow Size"] = "Shallow Size";
localizedStrings["# New"] = "# New";
localizedStrings["context var"] = "context var";
localizedStrings["system prop"] = "system prop";
localizedStrings["Property types:"] = "Property types:";
localizedStrings["element"] = "element";
localizedStrings["property"] = "property";
localizedStrings["Object types:"] = "Object types:";
localizedStrings["%s() at %s"] = "%s() at %s";
localizedStrings["Copy Stack Trace"] = "Copy Stack Trace";
localizedStrings["Collect Garbage"] = "Collect Garbage";
localizedStrings["to window objects"] = "to window objects";
localizedStrings["Can't find any paths."] = "Can't find any paths.";
localizedStrings["to GC roots"] = "to GC roots";
localizedStrings["Paths from the selected object"] = "Paths from the selected object";

