.help-window-outer {
    position: absolute;
    top: 60px;
    left: 5%;
    width: 90%;
    bottom: 40px;
    z-index: 2000;
}

body.attached .help-window-outer {
    top: 32px;
    left: 0;
    width: 100%;
    bottom: 24px;
}

.help-window-main {
    max-height: 100%;
    opacity: 0.85;
    color: white;
    background-color: black;
    display: -webkit-box;
    -webkit-box-orient: vertical;
    border: 20px black solid;
    border-top-width: 0;
    border-radius: 8px;
    -webkit-box-shadow: 10px 10px 8px rgba(40, 40, 40, 0.40);
}

body.attached .help-window-main {
    border-width: 10px;
    border-radius: 0;
    -webkit-box-shadow: 0 0 0;
}

.help-window-caption {
    margin: 8px;
}

body.attached .help-window-caption {
    display: none;
}

.help-window-title {
    border-bottom: solid 1px lightGrey;
    font-size: 18px;
    padding-bottom: 6px;
}

.help-content {
    overflow-y: auto;
    overflow-x: hidden;
    scrollbar-width: 11px;
    -webkit-box-flex: 1;
}

.help-content::-webkit-scrollbar {
    width: 11px;
}

.help-content::-webkit-scrollbar-corner,
.help-content::-webkit-resizer {
    display: none;
}

.help-content::-webkit-scrollbar-thumb:vertical {
    background: -webkit-gradient(linear, left top, right top, from(rgb(128, 128, 128)), to(rgb(128, 128, 128)), color-stop(40%, rgb(96, 96, 96)));
    border-radius: 5px;
    min-height: 20px;
}

.help-content::-webkit-scrollbar-thumb:vertical:hover,
.help-content::-webkit-scrollbar-thumb:vertical:active {
    background: -webkit-gradient(linear, left top, right top, from(rgb(176, 176, 176)), to(rgb(176, 176, 176)), color-stop(40%, rgb(144, 144, 144)));
}

.help-content::-webkit-scrollbar-track:vertical {
    background: -webkit-gradient(linear, left top, right top, from(rgb(10, 10, 10)), to(rgb(32, 32, 32)), color-stop(25%, rgb(32, 32, 32)));
    border-radius: 5px;
}

.help-close-button {
    border: 0;
    padding: 0;
    margin: 0px -20px 15px -20px;
    font-size: 14px;
    color: rgb(222, 222, 222);
    background: -webkit-gradient(radial, 30% 30%, 1, 50% 50%, 8, from(rgb(128, 128, 128)), to(rgb(80, 80, 80)));
    border-radius: 8px;
    height: 16px;
    width: 16px;
}

.help-close-button:hover {
    color: white;
}

body.platform-mac .help-close-button {
    float: left;
    margin-right: 10px;
    font-size: 12px;
}

body:not(.platform-mac) .help-close-button {
    float: right;
}

.help-table {
    width: 100%;
    font-size: 13px;
    -webkit-user-select: auto;
}

.help-table th {
    padding-top: 6px;
    text-align: left;
    color: yellow;
}

.help-table td {
    white-space: nowrap;
    vertical-align: top;
}

.help-key-cell {
    text-align: right;
}

.help-key {
    color: yellow;
}

body:not(.platform-mac) .help-key {
    font-weight: bold;
    text-shadow: black 1px 1px 7px;
}

body.platform-mac .help-key {
    font-family: Lucida Grande, sans-serif;
    font-size: 13px;
}

.help-combine-keys {
    color: white;
    font-weight: bold;
    margin: 0 0.3em;
    font-size: 12px;
}

.help-key-delimiter {
    color: white;
    margin: 0 0.5em;
}
