.go-to-line-dialog {
    position: absolute;
    top: 40%;
    left: 40%;
    z-index: 1900;

    background-image: -webkit-gradient(linear, 0% 0%, 0% 100%, from(#E9E9E9), to(#CFCFCF));
    display: -webkit-box;
    -webkit-box-orient: vertical;
    padding: 10px;
    border-radius: 10px;
    border: 1px solid gray;
    -webkit-box-shadow: rgb(40,40,40) 0px 0px 50px;

    font-size: 11px;
    font-family: 'Lucida Grande', sans-serif;
}

.go-to-line-dialog input {
    font-size: 11px;
}

.go-to-line-dialog button {
    font-size: 11px;
    color: rgb(6, 6, 6);
    border: 1px solid rgb(165, 165, 165);
    background-color: rgb(237, 237, 237);
    background-image: -webkit-gradient(linear, left top, left bottom, from(rgb(252, 252, 252)), to(rgb(223, 223, 223)));
    -webkit-border-radius: 12px;
    -webkit-appearance: none;

    padding: 3px 20px;
    margin: 0 0 0 10px;
}

.go-to-line-dialog button:active {
    background-color: rgb(215, 215, 215);
    background-image: -webkit-gradient(linear, left top, left bottom, from(rgb(194, 194, 194)), to(rgb(239, 239, 239)));
}
